function AuthController(OperatorService, OperatorController, WindowService, templateManager,
	ScreenService, UtilitiesService, PerifericosService, RegisterService, ZHPromise){

	var self = this;
	var modoMesa = "M";
	var modoComanda = "C";
	var modoBalcao = "B";
	var modoDelivery = "D";

	this.setAuthLogin = function (widget) {
		var email = widget.currentRow.EMAIL;
		widget.getField("PASSWORD").readOnly = widget.getField(
			"AUTH"
		).readOnly = !email;
		widget.activate();
	};

	this.validateIp = function(widget){
		var defer = ZHPromise.defer();

        if (projectConfig.saas){
            defer.resolve();
            templateManager.updateURL(
                "https://rc-odhenpos.teknisa.cloud/odhenPOS/backend/service/index.php"
            );

            return defer.promise;
        }
        else {
            return UtilitiesService.checkSetLocalVar().then(
                function (){
                    defer.resolve();
                },
                function () {
                    defer.reject();
                    ScreenService.showMessage("Configure o IP do servidor da aplicação.").then(function (){
                        UtilitiesService.prepareServerForm(widget.container.getWidget("serverIpWidget"));
                    });

                    return defer.promise;
                }
            );
        }
	};

	this.auth = function (widget) {
		this.validateIp(widget).then(function (){
			OperatorService.auth(widget.getField("EMAIL").value(), widget.getField("PASSWORD").value(),	projectConfig.frontVersion).then(function (response){
				if (!_.isEmpty(response[0].cdoperador)){
					WindowService.openWindow("LOGIN_FILIAL_SCREEN").then(function (){
						var widgetLogin = templateManager.container.getWidget("loginAuthWidget");
						widgetLogin.getField("OPERADOR").value(response[0].cdoperador);
						widgetLogin.getField("senha").value(widget.getField("PASSWORD").value());
						widget.getField("EMAIL").clearValue();
						widget.getField("PASSWORD").clearValue();
					});
				}
				else {
					ScreenService.confirmMessage(
						"A versão do servidor encontra-se desatualizada. Deseja atualizar?",
						"question",
						function (){
							ScreenService.confirmMessage(
								"AVISO IMPORTANTE:<br><br>Este procedimento poderá levar vários minutos, e não poderá ser cancelado. Favor aguarde até que a atualização termine antes de realizar qualquer operação.",
								"alert",
								function (){
									ScreenService.changeLoadingMessage("Atualizando a versão. Isto pode levar vários minutos. Aguarde...");
                                	OperatorService.updateVersion(projectConfig.frontVersion).then(
                                		function (updateResult){
                                    		ScreenService.restoreDefaultLoadingMessage();
                                    	},
                                    	function (){
                                    		ScreenService.restoreDefaultLoadingMessage();
                                    	}
                                    );
								},
								function (){}
							);
						},
						function (){}
					);
				}
			});
		});
	};

	this.handleFilialChange = function (filialField) {
		var widget = filialField.widget;
		self.getCaixasLogin(widget, filialField.widget.currentRow.CDFILIAL);
	};

	this.getCaixasLogin = function (widget, filial) {
		var caixaField = widget.getField("CAIXA");
		if (filial) {
			OperatorService.getCaixasLogin(filial, caixaField).then(function (caixas) {
				caixas = caixas.dataset.CaixasLogin || [];
				if (_.isEmpty(caixas)) {
					caixaField.readOnly = true;
					ScreenService.showMessage("Nenhum caixa encontrado na filial.");
				} else {
					caixaField.readOnly = false;
				}
				if (caixas.length == 1) {
					widget.setCurrentRow(_.merge(widget.currentRow, caixas[0]));
				} else {
					widget.currentRow.CAIXA = null;
					widget.currentRow.CDCAIXA = null;
				}
				caixaField.dataSource.data = caixas;
			});
		} else {
			caixaField.readOnly = true;
		}
	};

	this.authLogin = function (row) {
		var groupMenu = templateManager.project;
		var menus = groupMenu
			.getMenu("APLICACAO")
			.menus.concat(groupMenu.getMenu("CONSUMIDOR").menus);
		row.CDOPERADOR = row.OPERADOR;

		if (!row.CDFILIAL) throw "Informe a filial.";
		if (!row.CDCAIXA) throw "Informe o caixa.";
		if (!row.CDOPERADOR) throw "Informe o Operador.";
		if (!row.senha) throw "Informe a senha.";

		OperatorController.saveLoginData(row);

		OperatorService.login(
			row.CDFILIAL,
			row.CDCAIXA,
			row.CDOPERADOR,
			row.senha,
			projectConfig.frontVersion,
			projectConfig.currentMode
		).then(
			function (data) {
				if (data.OperatorRepository) {
					var operatorData = data.OperatorRepository[0];
					if (operatorData.paramsImpressora) {
						PerifericosService.test(operatorData.paramsImpressora).then(function (result) {
							if (!result.error) {
								self.handleLogin(operatorData, data, menus);
							} else {
								ScreenService.showMessage(result.message);
							}
						});
					} else {
						self.handleLogin(operatorData, data, menus);
					}
				} else {
					ScreenService.showMessage(data[0]);
				}
			}.bind(this),
			function () {
				throw "Erro na tentativa de login, verifique a configuração de IP.";
			}
		);
	};

	this.clearData = function(widget){
		widget.getField("CAIXA").clearValue();
		widget.getField("FILIAL").clearValue();
	};

	this.handleLogin = function (operatorData, data, menus) {
		OperatorController.checkSSLConnectionId(operatorData).then(
			function (checkSSLConnectionIdResult) {
				if (!checkSSLConnectionIdResult.error) {
					if (operatorData.IDCOLETOR !== "C") {
						var estadoCaixa = operatorData.estadoCaixa;
						var IDPALFUTRABRCXA = operatorData.IDPALFUTRABRCXA;
						var VRABERCAIX = operatorData.VRABERCAIX;
						var obrigaFechamento = operatorData.obrigaFechamento;
						if (estadoCaixa === "fechado" && IDPALFUTRABRCXA === "S") {
							OperatorController.bindedDoLogin = _.bind(
								OperatorController.doLogin,
								OperatorController,
								data,
								menus
							);
							WindowService.openWindow("OPEN_REGISTER_SCREEN");
						} else if (
							estadoCaixa === "fechado" &&
							IDPALFUTRABRCXA === "N"
						) {
							RegisterService.openRegister(
								operatorData.chave,
								VRABERCAIX
							).then(function () {
								OperatorController.doLogin(data, menus);
							});
						} else if (estadoCaixa === "aberto" && obrigaFechamento) {
							RegisterService.setClosingOnLogin(true);
							OperatorController.bindedDoLogin = _.bind(
								OperatorController.doLogin,
								OperatorController,
								data,
								menus
							);
							WindowService.openWindow("CLOSE_REGISTER_SCREEN");
						} else {
							OperatorController.doLogin(data, menus);
						}
					} else {
						if (operatorData.modoHabilitado === modoDelivery) {
							ScreenService.showMessage(
								"Modo delivery não pode ser coletor.",
								"alert"
							);
						} else {
							OperatorController.doLogin(data, menus);
						}
					}
				} else {
					ScreenService.showMessage(checkSSLConnectionIdResult.message);
				}
			}.bind(this)
		);
	};

	this.handleEnterButton = function (args){
		var keyCode = args.e.keyCode;
		if (keyCode === 13 || keyCode === 9){
			UtilitiesService.handleCloseKeyboard();
			var field = args.owner.field;
			var widget = field.widget;

			if (widget.name === "serverIpWidget") {
				if (field.name === "ip") {
					if (!Util.isDesktop()) document.getElementById("porta").focus();
				}
				else if (field.name === "porta") {
					UtilitiesService.setServerIp(widget.currentRow,	templateManager.container.getWidget("authWidget"));
				}
			}
		}
	};

}

Configuration(function (ContextRegister) {
	ContextRegister.register("AuthController", AuthController);
});
